#
# guimain.py
# $Id: guimain.py,v 1.8 1996/09/09 16:42:01 kzeil Exp $
#
# Koert Zeilstra - Network Solutions
# 1996.08.16
#

import sys
import string
import os
import tempfile

import Tkinter

import rwhois
import multicolumn
import errordlg
import preferdlg
import browser
import searchbrowse
import printdlg

class GuiRwhois(Tkinter.Toplevel):
  def __init__(self, parent):
    Tkinter.Toplevel.__init__(self, parent)

    self._main_ready = 0 # Indication to setStatus not to update
    self._query_string = Tkinter.StringVar()
    self._status_string = Tkinter.StringVar()
    self._query_string.set("")
    self._rc_filename = "%s/.pyrwhois/guirc" % os.environ["HOME"]
    self._server = None
    self._record_filter_menu = None
    self._record_filter_button = None
    self._filter_record = Tkinter.StringVar()
    self._field_menu = None
    self._field_button = None

    self._error_dlg = None
    self._prefer_dlg = None
    self._search_dialog = None
    self._print_dlg = None
    self._preferences = {"host": "rwhois.internic.net", "port": 4321,
			 "limit": 20}
    self._query_restrictor = Tkinter.StringVar()
    self._records_type_names = ["all"]
    self._schema_cache = rwhois.SchemaCache()
    self._current_schema = None
    self._query_history = []

    self.readRcFile(self._rc_filename)
    self.preferencesChanged()

    #self._record_type_names = ["all"] + self._server.getSchemaNames()
    self._record_type_names = "all",
    for name in self._server.getSchemaNames():
      t = name,
      self._record_type_names = self._record_type_names + t
    self.setStatus("%d schemas retrieved" % len(self._record_type_names))

    self.title('PyRwhois - 0.3')
    self.iconname('PyRwhois')

    menubar = Tkinter.Frame(self, {"relief": "raised", "bd":2, "class": "Menubar"})
    file = Tkinter.Menubutton (menubar, text="File", underline=0)
    file.pack (side="left", ipadx=4)
    file_menu = Tkinter.Menu(file)
    file_menu.add ('command', label="Preferences...", underline=0, 
		     command=self.menuFilePreferences)
    file_menu.add ('command', label="Print...", underline=0, 
		     command=self.menuFilePrint)
    file_menu.add ('separator')
    file_menu.add ('command', {'label': 'Quit', 'underline': 0, 
		     'command' : self.menuFileQuit})
    file['menu'] = file_menu

    results = Tkinter.Menubutton (menubar, {"text": "Results",
		"underline": 0})
    results.pack ({"side": "left", "ipadx": 4})
    results_menu = Tkinter.Menu(results)
    results_menu.add ("command", {"label": "Search", "underline": 0, 
		     "command": self.menuResultsSearch})
    results['menu'] = results_menu

    menubar.pack ({'side': 'top', 'fill': 'x'})

    query_frame = Tkinter.Frame(self)
    query_label = Tkinter.Label(query_frame, text="Query", width=8, anchor="w")
    query_label.pack(side="left")
    query_menu = apply(Tkinter.OptionMenu, (query_frame,
		 self._query_restrictor) + self._record_type_names)
    query_menu.pack(side="left")
    query_entry = Tkinter.Entry(query_frame, width=30, relief="sunken", bd=2,
		textvariable=self._query_string)
    query_entry.bind("<Return>", self.keyEventQuery)
    query_entry.pack(side="left", expand="yes", fill="x")
    query_button = Tkinter.Button(query_frame, text="Query",
		command=self.query)
    query_button.pack(side="left")
    clear_button = Tkinter.Button(query_frame, text="Clear",
		command=self.clearQuery)
    clear_button.pack(side="left")
    query_frame.pack(side="top", expand="no", fill="x")

    self._browser = browser.RecordBrowser(self)
    self._browser.queryCommand(self.referralQuery)
    self._browser.pack(side="top", fill="both", expand=1)

    self._status_entry = Tkinter.Entry(self, relief="sunken", bd=1,
		state="normal", textvariable=self._status_string)
    self._status_entry.pack(side="top", fill="x")
    self._main_ready = 1

    self.protocol("WM_DELETE_WINDOW", self.menuFileQuit)

  def menuFilePreferences(self):
    if self._prefer_dlg == None:
      self._prefer_dlg = preferdlg.PreferencesDialog(self)
      self._prefer_dlg.setPreferences(self._preferences)
      self._prefer_dlg.okCommand(self.okPreferencesDialog)
    else:
      self._prefer_dlg.deiconify()

  def okPreferencesDialog(self):
    self._preferences = self._prefer_dlg.getPreferences()
    self.writeRcFile(self._rc_filename)
    self.preferencesChanged()

  def menuFilePrint(self):
    if self._print_dlg == None:
      self._print_dlg = printdlg.PrintDialog(self)
      self._print_dlg.title("PyRwhois - Print")
      self._print_dlg.iconname("Print")
      self._print_dlg.closeCommand(self._print_dlg.withdraw)
      self._print_dlg.okCommand(self.okPrintDialog)
    else:
      self._print_dlg.deiconify()

  def okPrintDialog (self, print_command):
    filename = tempfile.mktemp()
    self._browser.printToFile(filename)
    os.system("%s %s" % (print_command, filename))

  def menuFileQuit(self):
    sys.exit(0)

  def menuResultsSearch(self):
    if self._search_dialog == None:
      self._search_dialog = searchbrowse.SearchBrowser(self)
      self._search_dialog.title("PyRwhois - Search Results")
      self._search_dialog.iconname("Search Results")
      self._search_dialog.closeCommand(self._search_dialog.withdraw)
      self._search_dialog.findCommand(self.searchResults)
      self._search_dialog.queryCommand(self.referralQuery)
    else:
      self._search_dialog.deiconify()

  def keyEventQuery(self, event):
    self.query()

  def query(self):
    query_text = self._query_string.get()
    restrictor = self._query_restrictor.get()
    if restrictor != "all":
      query_text = "%s %s" % (restrictor, query_text)
    self.makeQuery(self._server.getHost(), self._server.getPort(), query_text)

  def getUniqueItems(self, list):
    new_list = []
    for item in list:
      if not item in new_list:
        new_list.append(item)
    return new_list

  def readRcFile(self, filename):
    try:
      preferences = {}
      fp = open(filename, "r")
      done = 0
      while done == 0:
        line = fp.readline()
        if not line:
          done = 1
        else:
          items = string.splitfields(line, ":")
          id = string.strip(items[0])
          preferences[id] = string.strip(items[1])
      fp.close()
      preferences["port"] = string.atoi(preferences["port"])
      preferences["limit"] = string.atoi(preferences["limit"])
      self._preferences = preferences
    except IOError:
      pass

  def writeRcFile(self, filename):
    try:
      fp = open(filename, "w")
      config_list = self._preferences.keys()
      for id in config_list:
        fp.write("%s:%s\n" % (id, self._preferences[id]))
      fp.close()
    except IOError:
      pass

  def preferencesChanged(self):
    host = self._preferences["host"]
    port = self._preferences["port"]
    self._server = rwhois.Rwhois(host, port)
    self._server.setFeedbackCommand(self.setStatus)
    self._server.setLimit(self._preferences["limit"])

  def setStatus(self, message):
    self._status_string.set(message)
    if self._main_ready:
      self.update()

  def clearQuery(self):
    self._query_string.set("")

  def makeQuery(self, host, port, query):
    self.update()
    self._records, errors = self._server.query(query)
    if len(errors) > 0:
      errors = self.getUniqueItems(errors)
      if self._error_dlg == None:
        self._error_dlg = errordlg.ErrorDialog(self)
      self._error_dlg.setMessage(string.join(errors, "\n"))
      self._error_dlg.deiconify()

    self._query_record_types = []
    for record in self._records:
      if not record.getSchemaName() in self._query_record_types:
        self._query_record_types.append(record.getSchemaName())
      #self._records_list.insert("end", record.getField("Handle"))
    no_records = len(self._records)
    if no_records > 0:
      if no_records == 1:
        self.setStatus("1 record retrieved")
      else:
        self.setStatus("%d records retrieved" % no_records)

    self._browser.showRecords(self._records, self._schema_cache)

  def referralQuery(self, host, port, query):
    records, errors = self._server.query(query)
    if len(errors) > 0:
      errors = self.getUniqueItems(errors)
      if self._error_dlg == None:
        self._error_dlg = errordlg.ErrorDialog(self)
      self._error_dlg.setMessage(string.join(errors, "\n"))
      self._error_dlg.deiconify()

    no_records = len(records)
    if no_records > 0:
      if no_records == 1:
        self.setStatus("1 record retrieved")
      else:
        self.setStatus("%d records retrieved" % no_records)

    return records

  def searchResults (self, search_string, ignore_case):
    found_list = []
    for record in self._records:
      value_lists = record.getFieldValues()
      found = 0
      for value_list in value_lists:
        for value in value_list:
          if string.find(value, search_string) != -1:
            found = 1
      if found:
        found_list.append(record)
    self._search_dialog.showRecords(found_list, self._schema_cache)

dir_name = "%s/.pyrwhois" % os.environ["HOME"]
if not os.path.exists(dir_name):
  os.mkdir(dir_name, 0700)

root = Tkinter.Tk()
root.withdraw()

#root.option_add("*background", "DarkSeaGreen", "widgetDefault")
#root.option_add("*highlightBackground", "DarkSeaGreen", "widgetDefault")
root.option_add("*font", "-*-lucida-medium-r-normal-*-12-*-*-*-*-*-*-*", "widgetDefault")
root.option_add("*Text*font", "-*-lucidatypewriter-medium-r-normal-*-12-*-*-*-*-*-*-*", "widgetDefault")
root.option_add("*Menubar*font", "-*-helvetica-bold-r-normal-*-*-140-75-75-*-*-*-*", "widgetDefault")
#root.option_add("*Menubar*activeForeground", "Blue", "widgetDefault")
#root.option_add("*Menubar*activeBackground", "DarkSeaGreen", "widgetDefault")
root.option_add("*Button.background", "Gray80", "widgetDefault")
root.option_add("*Button.activeBackground", "Gray90", "widgetDefault")

app = GuiRwhois(root)
root.mainloop ()
